#!/bin/sh

# Package sd card wic images into zip files
# Include md5sum file and bmap file
# Place bmap in .meta folder to support balenaEtcher
# NOTE: Command is expected to be run from deploy/images/... folder and looks in current PWD
# Changelog:
# 07/2021: Create symlinks to .img.zip files to save time

## zip isn't installed in crops/poky image so use yocto native built one
## BUILDDIR is set by yocto
export PATH=$PATH:$BUILDDIR/arago-tmp-external-arm-toolchain/sysroots-components/x86_64/zip-native/usr/bin/
if ! command -v zip > /dev/null
then
	echo "ERROR: Can't find zip binary"
	exit 1
fi

# HACK: balencaEtcher requires image files to be a multiple of the bmap block size (4KiB)
# https://github.com/balena-io/etcher/issues/3475
# Add some padding to image files so they are divisible by 4096B
blockSize=4096
echo "Padding image files to be multiple of ${blockSize}B"
for f in *.wic; do
	if [ -L "$f" ]; then
		continue; # Skip symlinks
	fi

	# Round up file size by $blockSize
	fileSizeB=$(stat --format=%s "$f")
	fileSizeRoundedUpBlocks=$(( (fileSizeB + (blockSize-1))/blockSize ))
	fileSizeRoundedUpBytes=$(( fileSizeRoundedUpBlocks * blockSize ))

	if [ "$fileSizeB" != "$fileSizeRoundedUpBytes" ]; then

		# Increase image size
		truncate -s "${fileSizeRoundedUpBytes}" "$f"

		fileSizeBAfter=$(stat --format=%s "$f")

		# Update bmap file
		bmaptool create "$f" -o "$f".bmap

		echo File $f size before $fileSizeB and after $fileSizeBAfter
	fi
done

# Rename wic to .img and zip up sd card images
for f in *.wic; do

	# Replace .wic with .img
	img="$(basename -s .wic "$f").img"

	if [ -L "$f" ]; then
		# Create symlink to created .img.zip file
		target_zip=$(basename -s .wic $(readlink -f "$f")).img.zip

		ln -sfv $target_zip "${img}.zip"
		continue; # Skip symlinks
	fi

	cp "$f" "$img"
	cp "$f".bmap "$img".bmap

	# Cleanup leftover .meta folders
	rm -rf .meta
	# Create .meta folder for balenaEtcher bmap support
	mkdir .meta
	cp "$f".bmap .meta/image.bmap

	md5sum "$img" > "$img.md5"
	zip -r "$img".zip "$img"* .meta

	# Cleanup .meta folders
	rm -rf .meta
	# Cleanup img files
	rm "$img".bmap "$img" "$img".md5
done
